/*
Copyright (c) 2001 David Drysdale

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

Except as contained in this notice, the name of the author shall not be
used in advertising or otherwise to promote the sale, use or other dealings
in this Software without prior written authorization from the author.
*/

import java.awt.*;
import java.net.InetAddress;
import java.util.Date;

/*
 * GridCanvas - extends Canvas to allow drawing of the gridfont
 */
public class GridCanvas extends Canvas {
    static final int leftMargin = 10;
    static final int interColSpace = 5;
    static final int interRowSpace = 7;
    static final int numCols = 13;
    static final int numRows = 2;

    boolean inited = false;
    GridFont owner;
    GridLetter alphabet[][];
    GridLetter currentFocus = null;
    // The maximum space that each letter can occupy
    int letterWidth;
    int letterHeight;
    // The actual space that each letter does occupy
    int actualLetterWidth;
    int actualLetterHeight;
    // Clipboard for quanta
    boolean clippedQuanta[][][];

    FontDescription descr;

    public GridCanvas(GridFont gf) {
        owner = gf;
        clippedQuanta = new boolean[owner.numHDivs+1][owner.numVDivs+1][4];
        descr = new FontDescription();
    }
    public void init() {
        if (owner.debugMode) System.out.println("Initialize GridCanvas instance");
        Graphics g = getGraphics();
        // Figure out the relevant sizes
        letterWidth  = ((size().width - leftMargin)/numCols) - interColSpace;
        letterHeight = (size().height/numRows) - interRowSpace;

        alphabet = new GridLetter[numCols][numRows];
        char letter = 'a';
        for (int jj=0; jj<numRows; jj++) {
            for (int ii=0; ii<numCols; ii++) {
                alphabet[ii][jj] = new 
                    GridLetter(owner, this, g,
                               (interColSpace/2) + ii*(letterWidth +interColSpace) + leftMargin,
                               (interRowSpace/2) + jj*(letterHeight+interRowSpace),
                               letterWidth,
                               letterHeight,
                               letter);
                if (letter != ' ') letter++;
                if (letter > 'z') letter = ' ';
            }
        }
        // Use the actual dimensions of each letter
        actualLetterWidth = alphabet[0][0].getHSize();
        actualLetterHeight = alphabet[0][0].getVSize();
        inited = true;
        if (owner.debugMode) System.out.println("..finished init");
    }

    public boolean mouseDown(Event e, int x, int y) {
        int ii = -1;
        int jj = -1;
        int adjustedx = x - ((interColSpace/2) + leftMargin);
        if (adjustedx > 0) {
            ii = adjustedx / (letterWidth+interColSpace);
            if (ii >= numCols) ii = -1;
            // Also check that it is not between cols
            else if ((adjustedx - ii*(letterWidth+interColSpace)) > actualLetterWidth) ii = -1;
        }
        int adjustedy = y - (interRowSpace/2);
        if (adjustedy > 0) {
            jj = adjustedy / (letterHeight+interRowSpace);
            if (jj >= numRows) jj = -1;
            // Also check that it is not between rows
            else if ((adjustedy - jj*(letterHeight+interRowSpace)) > actualLetterHeight) jj = -1;
        }
        if ((ii!=-1) && (jj!=-1)) {
            // Set up the new focus, removing any old focus
            GridLetter newFocus = alphabet[ii][jj];
            if ((newFocus!=currentFocus) && (currentFocus != null)) {
                currentFocus.lostFocus();
            }
            newFocus.gainedFocus();
            currentFocus = newFocus;
            // given that the input was within the actual bounds of this letter,
            // pass the input on to the letter for processing
            newFocus.mouseDown(x, y);
            newFocus.draw(getGraphics());
        } else {
            // Remove the old focus
            if (currentFocus != null) currentFocus.lostFocus();
            currentFocus = null;
        }
        return true;
    }

    // Methods to clear all or just the current letter
    public void clearCurrent() {
        if (currentFocus != null) {
            currentFocus.clear();
            currentFocus.draw(getGraphics());
        }
    }
    public void clearAll() {
        for (int ii=0; ii<numCols; ii++) {
            for (int jj=0; jj<numRows; jj++) {
                alphabet[ii][jj].clear();
            }
        }
        descr = new FontDescription();
        paint(getGraphics());
    }

    // Methods to copy/paste the current letter's quanta
    public void copyCurrent() {
        if (currentFocus != null) {
            currentFocus.getQuanta(clippedQuanta);
        }
    }
    public void pasteCurrent() {
        if (currentFocus != null) {
            currentFocus.setQuanta(clippedQuanta);
            currentFocus.draw(getGraphics());
        }
    }
    // Methods to transform current letter
    public void vflipCurrent() {
        if (currentFocus != null) {
            currentFocus.vflip();
            currentFocus.draw(getGraphics());
        }
    }
    public void hflipCurrent() {
        if (currentFocus != null) {
            currentFocus.hflip();
            currentFocus.draw(getGraphics());
        }
    }
    public void rotateCurrent() {
        if (currentFocus != null) {
            currentFocus.rotate();
            currentFocus.draw(getGraphics());
        }
    }

    // Set up all the letters from the passed font string
    public void setFontString(String font) {
        descr = new FontDescription(font);
        owner.setFontName(descr.fontName);
        char letter = 'a';
        for (int jj=0; jj<numRows; jj++) {
            for (int ii=0; ii<numCols; ii++) {
                if (descr.letterString[letter - 'a'] != null) {
                    alphabet[ii][jj].setEncoding(descr.letterString[letter - 'a']);
                } else {
                    alphabet[ii][jj].clear();
                }
                if (letter != ' ') letter++;
                if (letter > 'z') return;
            }
        }
    }

    // Return a text description of the font
    public String getFontString(String fontName) {
        Date now = new Date();
        String fontDesc = 
            "font : " + fontName + "\n" +
            "creator : " + descr.creatorString + "\n" +
            "create date : " + descr.createDate.toString() + "\n" +
            "last edit : " + now.toString() + "\n";
        char letter = 'a';
        outer: for (int jj=0; jj<numRows; jj++) {
            for (int ii=0; ii<numCols; ii++) {
                fontDesc = fontDesc + alphabet[ii][jj].getEncodingString() + "\n";
                if (letter != ' ') letter++;
                if (letter > 'z') break outer;
            }
        }
        fontDesc = fontDesc + "comment : " + "\n";
        return fontDesc;
    }

    // Display the board
    public void paint(Graphics g) {
        if (!inited) return; // dont draw till initialized
        for (int ii=0; ii<numCols; ii++) {
            for (int jj=0; jj<numRows; jj++) {
                alphabet[ii][jj].draw(g);
            }
        }
    }

}
